@@ -13,15 +13,14 @@ auditTest(
     function(m) {
         testDB = m.getDB(testDBName);
         assert.writeOK(testDB.getCollection('foo').insert({ a: 1 }));
-        const beforeCmd = Date.now();
         assert.commandWorked(testDB.dropDatabase());
 
-        const beforeLoad = Date.now();
+        beforeLoad = Date.now();
         var auditColl = getAuditEventsCollection(m, testDBName);
         assert.eq(1, auditColl.count({
             atype: "dropDatabase",
-            ts: withinInterval(beforeCmd, beforeLoad),
-            'param.ns': testDBName,
+            ts: withinFewSecondsBefore(beforeLoad),
+            'params.ns': testDBName,
             result: 0,
         }), "FAILED, audit log: " + tojson(auditColl.find().toArray()));
     },
