@@ -1,21 +1,16 @@
-// Copyright (C) MongoDB, Inc. 2014-present.
-//
-// Licensed under the Apache License, Version 2.0 (the "License"); you may
-// not use this file except in compliance with the License. You may obtain
-// a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
-
 package mongoimport
 
 import (
+	"fmt"
+	"github.com/mongodb/mongo-tools/common/db"
+	"github.com/mongodb/mongo-tools/common/log"
+	"github.com/mongodb/mongo-tools/common/options"
+	"github.com/mongodb/mongo-tools/common/testutil"
+	. "github.com/smartystreets/goconvey/convey"
+	"gopkg.in/mgo.v2/bson"
+	"gopkg.in/tomb.v2"
 	"io"
 	"testing"
-
-	"github.com/mongodb/mongo-tools-common/log"
-	"github.com/mongodb/mongo-tools-common/options"
-	"github.com/mongodb/mongo-tools-common/testtype"
-	. "github.com/smartystreets/goconvey/convey"
-	"go.mongodb.org/mongo-driver/bson"
-	"gopkg.in/tomb.v2"
 )
 
 func init() {
@@ -27,73 +22,57 @@ func init() {
 var (
 	index         = uint64(0)
 	csvConverters = []CSVConverter{
-		{
-			colSpecs: []ColumnSpec{
-				{"field1", new(FieldAutoParser), pgAutoCast, "auto", []string{"field1"}},
-				{"field2", new(FieldAutoParser), pgAutoCast, "auto", []string{"field2"}},
-				{"field3", new(FieldAutoParser), pgAutoCast, "auto", []string{"field3"}},
-			},
-			data:  []string{"a", "b", "c"},
-			index: index,
+		CSVConverter{
+			fields: []string{"field1", "field2", "field3"},
+			data:   []string{"a", "b", "c"},
+			index:  index,
 		},
-		{
-			colSpecs: []ColumnSpec{
-				{"field4", new(FieldAutoParser), pgAutoCast, "auto", []string{"field4"}},
-				{"field5", new(FieldAutoParser), pgAutoCast, "auto", []string{"field5"}},
-				{"field6", new(FieldAutoParser), pgAutoCast, "auto", []string{"field6"}},
-			},
-			data:  []string{"d", "e", "f"},
-			index: index,
+		CSVConverter{
+			fields: []string{"field4", "field5", "field6"},
+			data:   []string{"d", "e", "f"},
+			index:  index,
 		},
-		{
-			colSpecs: []ColumnSpec{
-				{"field7", new(FieldAutoParser), pgAutoCast, "auto", []string{"field7"}},
-				{"field8", new(FieldAutoParser), pgAutoCast, "auto", []string{"field8"}},
-				{"field9", new(FieldAutoParser), pgAutoCast, "auto", []string{"field9"}},
-			},
-			data:  []string{"d", "e", "f"},
-			index: index,
+		CSVConverter{
+			fields: []string{"field7", "field8", "field9"},
+			data:   []string{"d", "e", "f"},
+			index:  index,
 		},
-		{
-			colSpecs: []ColumnSpec{
-				{"field10", new(FieldAutoParser), pgAutoCast, "auto", []string{"field10"}},
-				{"field11", new(FieldAutoParser), pgAutoCast, "auto", []string{"field11"}},
-				{"field12", new(FieldAutoParser), pgAutoCast, "auto", []string{"field12"}},
-			},
-			data:  []string{"d", "e", "f"},
-			index: index,
+		CSVConverter{
+			fields: []string{"field10", "field11", "field12"},
+			data:   []string{"d", "e", "f"},
+			index:  index,
 		},
-		{
-			colSpecs: []ColumnSpec{
-				{"field13", new(FieldAutoParser), pgAutoCast, "auto", []string{"field13"}},
-				{"field14", new(FieldAutoParser), pgAutoCast, "auto", []string{"field14"}},
-				{"field15", new(FieldAutoParser), pgAutoCast, "auto", []string{"field15"}},
-			},
-			data:  []string{"d", "e", "f"},
-			index: index,
+		CSVConverter{
+			fields: []string{"field13", "field14", "field15"},
+			data:   []string{"d", "e", "f"},
+			index:  index,
 		},
 	}
 	expectedDocuments = []bson.D{
-		{
-			{"field1", "a"},
-			{"field2", "b"},
-			{"field3", "c"},
-		}, {
-			{"field4", "d"},
-			{"field5", "e"},
-			{"field6", "f"},
-		}, {
-			{"field7", "d"},
-			{"field8", "e"},
-			{"field9", "f"},
-		}, {
-			{"field10", "d"},
-			{"field11", "e"},
-			{"field12", "f"},
-		}, {
-			{"field13", "d"},
-			{"field14", "e"},
-			{"field15", "f"},
+		bson.D{
+			bson.DocElem{"field1", "a"},
+			bson.DocElem{"field2", "b"},
+			bson.DocElem{"field3", "c"},
+		},
+		bson.D{
+			bson.DocElem{"field4", "d"},
+			bson.DocElem{"field5", "e"},
+			bson.DocElem{"field6", "f"},
+		},
+		bson.D{
+			bson.DocElem{"field7", "d"},
+			bson.DocElem{"field8", "e"},
+			bson.DocElem{"field9", "f"},
+		},
+		bson.D{
+			bson.DocElem{"field10", "d"},
+			bson.DocElem{"field11", "e"},
+			bson.DocElem{"field12", "f"},
+		},
+		bson.D{
+			bson.DocElem{"field13", "d"},
+			bson.DocElem{"field14", "e"},
+			bson.DocElem{"field15", "f"},
 		},
 	}
 )
@@ -103,49 +82,49 @@ func convertBSONDToRaw(documents []bson.D) []bson.Raw {
 	for _, document := range documents {
 		rawBytes, err := bson.Marshal(document)
 		So(err, ShouldBeNil)
-		rawBSONDocuments = append(rawBSONDocuments, rawBytes)
+		rawBSONDocuments = append(rawBSONDocuments, bson.Raw{3, rawBytes})
 	}
 	return rawBSONDocuments
 }
 
 func TestValidateFields(t *testing.T) {
-	testtype.SkipUnlessTestType(t, testtype.UnitTestType)
+	testutil.VerifyTestType(t, testutil.UnitTestType)
 
 	Convey("Given an import input, in validating the headers", t, func() {
 		Convey("if the fields contain '..', an error should be thrown", func() {
-			So(validateFields([]string{"a..a"}, false), ShouldNotBeNil)
+			So(validateFields([]string{"a..a"}), ShouldNotBeNil)
 		})
 		Convey("if the fields start/end in a '.', an error should be thrown", func() {
-			So(validateFields([]string{".a"}, false), ShouldNotBeNil)
-			So(validateFields([]string{"a."}, false), ShouldNotBeNil)
+			So(validateFields([]string{".a"}), ShouldNotBeNil)
+			So(validateFields([]string{"a."}), ShouldNotBeNil)
 		})
 		Convey("if the fields start in a '$', an error should be thrown", func() {
-			So(validateFields([]string{"$.a"}, false), ShouldNotBeNil)
-			So(validateFields([]string{"$"}, false), ShouldNotBeNil)
-			So(validateFields([]string{"$a"}, false), ShouldNotBeNil)
-			So(validateFields([]string{"a$a"}, false), ShouldBeNil)
+			So(validateFields([]string{"$.a"}), ShouldNotBeNil)
+			So(validateFields([]string{"$"}), ShouldNotBeNil)
+			So(validateFields([]string{"$a"}), ShouldNotBeNil)
+			So(validateFields([]string{"a$a"}), ShouldBeNil)
 		})
 		Convey("if the fields collide, an error should be thrown", func() {
-			So(validateFields([]string{"a", "a.a"}, false), ShouldNotBeNil)
-			So(validateFields([]string{"a", "a.ba", "b.a"}, false), ShouldNotBeNil)
-			So(validateFields([]string{"a", "a.ba", "b.a"}, false), ShouldNotBeNil)
-			So(validateFields([]string{"a", "a.b.c"}, false), ShouldNotBeNil)
+			So(validateFields([]string{"a", "a.a"}), ShouldNotBeNil)
+			So(validateFields([]string{"a", "a.ba", "b.a"}), ShouldNotBeNil)
+			So(validateFields([]string{"a", "a.ba", "b.a"}), ShouldNotBeNil)
+			So(validateFields([]string{"a", "a.b.c"}), ShouldNotBeNil)
 		})
 		Convey("if the fields don't collide, no error should be thrown", func() {
-			So(validateFields([]string{"a", "aa"}, false), ShouldBeNil)
-			So(validateFields([]string{"a", "aa", "b.a", "b.c"}, false), ShouldBeNil)
-			So(validateFields([]string{"a", "ba", "ab", "b.a"}, false), ShouldBeNil)
-			So(validateFields([]string{"a", "ba", "ab", "b.a", "b.c.d"}, false), ShouldBeNil)
-			So(validateFields([]string{"a", "ab.c"}, false), ShouldBeNil)
+			So(validateFields([]string{"a", "aa"}), ShouldBeNil)
+			So(validateFields([]string{"a", "aa", "b.a", "b.c"}), ShouldBeNil)
+			So(validateFields([]string{"a", "ba", "ab", "b.a"}), ShouldBeNil)
+			So(validateFields([]string{"a", "ba", "ab", "b.a", "b.c.d"}), ShouldBeNil)
+			So(validateFields([]string{"a", "ab.c"}), ShouldBeNil)
 		})
 		Convey("if the fields contain the same keys, an error should be thrown", func() {
-			So(validateFields([]string{"a", "ba", "a"}, false), ShouldNotBeNil)
+			So(validateFields([]string{"a", "ba", "a"}), ShouldNotBeNil)
 		})
 	})
 }
 
 func TestGetUpsertValue(t *testing.T) {
-	testtype.SkipUnlessTestType(t, testtype.UnitTestType)
+	testutil.VerifyTestType(t, testutil.UnitTestType)
 
 	Convey("Given a field and a BSON document, on calling getUpsertValue", t, func() {
 		Convey("the value of the key should be correct for unnested documents", func() {
@@ -153,13 +132,7 @@ func TestGetUpsertValue(t *testing.T) {
 			So(getUpsertValue("a", bsonDocument), ShouldEqual, 3)
 		})
 		Convey("the value of the key should be correct for nested document fields", func() {
-			inner := bson.D{{"b", 4}}
-			bsonDocument := bson.D{{"a", inner}}
-			So(getUpsertValue("a.b", bsonDocument), ShouldEqual, 4)
-		})
-		Convey("the value of the key should be correct for nested document pointer fields", func() {
-			inner := bson.D{{"b", 4}}
-			bsonDocument := bson.D{{"a", &inner}}
+			bsonDocument := bson.D{{"a", bson.D{{"b", 4}}}}
 			So(getUpsertValue("a.b", bsonDocument), ShouldEqual, 4)
 		})
 		Convey("the value of the key should be nil for unnested document "+
@@ -169,14 +142,7 @@ func TestGetUpsertValue(t *testing.T) {
 		})
 		Convey("the value of the key should be nil for nested document "+
 			"fields that do not exist", func() {
-			inner := bson.D{{"b", 4}}
-			bsonDocument := bson.D{{"a", inner}}
-			So(getUpsertValue("a.c", bsonDocument), ShouldBeNil)
-		})
-		Convey("the value of the key should be nil for nested document pointer "+
-			"fields that do not exist", func() {
-			inner := bson.D{{"b", 4}}
-			bsonDocument := bson.D{{"a", &inner}}
+			bsonDocument := bson.D{{"a", bson.D{{"b", 4}}}}
 			So(getUpsertValue("a.c", bsonDocument), ShouldBeNil)
 		})
 		Convey("the value of the key should be nil for nil document values", func() {
@@ -186,7 +152,7 @@ func TestGetUpsertValue(t *testing.T) {
 }
 
 func TestConstructUpsertDocument(t *testing.T) {
-	testtype.SkipUnlessTestType(t, testtype.UnitTestType)
+	testutil.VerifyTestType(t, testutil.UnitTestType)
 
 	Convey("Given a set of upsert fields and a BSON document, on calling "+
 		"constructUpsertDocument", t, func() {
@@ -209,8 +175,7 @@ func TestConstructUpsertDocument(t *testing.T) {
 		})
 		Convey("the key/value combination in the upsert document should be "+
 			"correct for nested documents with several fields", func() {
-			inner := bson.D{{testCollection, 4}}
-			bsonDocument := bson.D{{"a", inner}, {"b", "string value"}}
+			bsonDocument := bson.D{{"a", bson.D{{testCollection, 4}}}, {"b", "string value"}}
 			upsertFields := []string{"a.c"}
 			expectedDocument := bson.D{{"a.c", 4}}
 			upsertDocument := constructUpsertDocument(upsertFields,
@@ -227,51 +192,66 @@ func TestConstructUpsertDocument(t *testing.T) {
 	})
 }
 
-func TestSetNestedDocumentValue(t *testing.T) {
-	testtype.SkipUnlessTestType(t, testtype.UnitTestType)
+func TestGetParsedValue(t *testing.T) {
+	testutil.VerifyTestType(t, testutil.UnitTestType)
+
+	Convey("Given a string token to parse", t, func() {
+		Convey("an int token should return the underlying int value", func() {
+			So(getParsedValue("3"), ShouldEqual, 3)
+		})
+		Convey("a float token should return the underlying float value", func() {
+			So(getParsedValue(".33"), ShouldEqual, 0.33)
+		})
+		Convey("a string token should return the underlying string value", func() {
+			So(getParsedValue("sd"), ShouldEqual, "sd")
+		})
+	})
+}
+
+func TestSetNestedValue(t *testing.T) {
+	testutil.VerifyTestType(t, testutil.UnitTestType)
 
 	Convey("Given a field, its value, and an existing BSON document...", t, func() {
-		b := bson.D{{"c", "d"}}
 		currentDocument := bson.D{
-			{"a", 3},
-			{"b", &b},
+			bson.DocElem{"a", 3},
+			bson.DocElem{"b", &bson.D{bson.DocElem{"c", "d"}}},
 		}
 		Convey("ensure top level fields are set and others, unchanged", func() {
 			testDocument := &currentDocument
-			expectedDocument := bson.E{"c", 4}
-			setNestedDocumentValue([]string{"c"}, 4, testDocument, false)
+			expectedDocument := bson.DocElem{"c", 4}
+			setNestedValue("c", 4, testDocument)
 			newDocument := *testDocument
 			So(len(newDocument), ShouldEqual, 3)
 			So(newDocument[2], ShouldResemble, expectedDocument)
 		})
 		Convey("ensure new nested top-level fields are set and others, unchanged", func() {
 			testDocument := &currentDocument
-			expectedDocument := bson.D{{"b", "4"}}
-			setNestedDocumentValue([]string{"c", "b"}, "4", testDocument, false)
+			expectedDocument := bson.D{bson.DocElem{"b", "4"}}
+			setNestedValue("c.b", "4", testDocument)
 			newDocument := *testDocument
 			So(len(newDocument), ShouldEqual, 3)
-			So(newDocument[2].Key, ShouldResemble, "c")
+			So(newDocument[2].Name, ShouldResemble, "c")
 			So(*newDocument[2].Value.(*bson.D), ShouldResemble, expectedDocument)
 		})
 		Convey("ensure existing nested level fields are set and others, unchanged", func() {
 			testDocument := &currentDocument
-			expectedDocument := bson.D{{"c", "d"}, {"d", 9}}
-			setNestedDocumentValue([]string{"b", "d"}, 9, testDocument, false)
+			expectedDocument := bson.D{bson.DocElem{"c", "d"}, bson.DocElem{"d", 9}}
+			setNestedValue("b.d", 9, testDocument)
 			newDocument := *testDocument
 			So(len(newDocument), ShouldEqual, 2)
-			So(newDocument[1].Key, ShouldResemble, "b")
+			So(newDocument[1].Name, ShouldResemble, "b")
 			So(*newDocument[1].Value.(*bson.D), ShouldResemble, expectedDocument)
 		})
 		Convey("ensure subsequent calls update fields accordingly", func() {
 			testDocument := &currentDocument
-			expectedDocumentOne := bson.D{{"c", "d"}, {"d", 9}}
-			expectedDocumentTwo := bson.E{"f", 23}
-			setNestedDocumentValue([]string{"b", "d"}, 9, testDocument, false)
+			expectedDocumentOne := bson.D{bson.DocElem{"c", "d"}, bson.DocElem{"d", 9}}
+			expectedDocumentTwo := bson.DocElem{"f", 23}
+			setNestedValue("b.d", 9, testDocument)
 			newDocument := *testDocument
 			So(len(newDocument), ShouldEqual, 2)
-			So(newDocument[1].Key, ShouldResemble, "b")
+			So(newDocument[1].Name, ShouldResemble, "b")
 			So(*newDocument[1].Value.(*bson.D), ShouldResemble, expectedDocumentOne)
-			setNestedDocumentValue([]string{"f"}, 23, testDocument, false)
+			setNestedValue("f", 23, testDocument)
 			newDocument = *testDocument
 			So(len(newDocument), ShouldEqual, 3)
 			So(newDocument[2], ShouldResemble, expectedDocumentTwo)
@@ -280,36 +260,33 @@ func TestSetNestedDocumentValue(t *testing.T) {
 }
 
 func TestRemoveBlankFields(t *testing.T) {
-	testtype.SkipUnlessTestType(t, testtype.UnitTestType)
+	testutil.VerifyTestType(t, testutil.UnitTestType)
 
 	Convey("Given an unordered BSON document", t, func() {
 		Convey("the same document should be returned if there are no blanks", func() {
-			bsonDocument := bson.D{{"a", 3}, {"b", "hello"}}
+			bsonDocument := bson.D{bson.DocElem{"a", 3}, bson.DocElem{"b", "hello"}}
 			So(removeBlankFields(bsonDocument), ShouldResemble, bsonDocument)
 		})
 		Convey("a new document without blanks should be returned if there are "+
 			" blanks", func() {
-			d := bson.D{
-				{"a", ""},
-				{"b", ""},
-			}
-			e := bson.D{
-				{"a", ""},
-				{"b", 1},
-			}
 			bsonDocument := bson.D{
-				{"a", 0},
-				{"b", ""},
-				{"c", ""},
-				{"d", &d},
-				{"e", &e},
-			}
-			inner := bson.D{
-				{"b", 1},
+				bson.DocElem{"a", 0},
+				bson.DocElem{"b", ""},
+				bson.DocElem{"c", ""},
+				bson.DocElem{"d", &bson.D{
+					bson.DocElem{"a", ""},
+					bson.DocElem{"b", ""},
+				}},
+				bson.DocElem{"e", &bson.D{
+					bson.DocElem{"a", ""},
+					bson.DocElem{"b", 1},
+				}},
 			}
 			expectedDocument := bson.D{
-				{"a", 0},
-				{"e", inner},
+				bson.DocElem{"a", 0},
+				bson.DocElem{"e", bson.D{
+					bson.DocElem{"b", 1},
+				}},
 			}
 			So(removeBlankFields(bsonDocument), ShouldResemble, expectedDocument)
 		})
@@ -317,116 +294,91 @@ func TestRemoveBlankFields(t *testing.T) {
 }
 
 func TestTokensToBSON(t *testing.T) {
-	testtype.SkipUnlessTestType(t, testtype.UnitTestType)
+	testutil.VerifyTestType(t, testutil.UnitTestType)
 
-	Convey("Given an slice of column specs and tokens to convert to BSON", t, func() {
-		Convey("the expected ordered BSON should be produced for the given"+
-			"column specs and tokens", func() {
-			colSpecs := []ColumnSpec{
-				{"a", new(FieldAutoParser), pgAutoCast, "auto", []string{"a"}},
-				{"b", new(FieldAutoParser), pgAutoCast, "auto", []string{"b"}},
-				{"c", new(FieldAutoParser), pgAutoCast, "auto", []string{"c"}},
-			}
+	Convey("Given an slice of fields and tokens to convert to BSON", t, func() {
+		Convey("the expected ordered BSON should be produced for the fields/tokens given", func() {
+			fields := []string{"a", "b", "c"}
 			tokens := []string{"1", "2", "hello"}
 			expectedDocument := bson.D{
-				{"a", int32(1)},
-				{"b", int32(2)},
-				{"c", "hello"},
+				bson.DocElem{"a", 1},
+				bson.DocElem{"b", 2},
+				bson.DocElem{"c", "hello"},
 			}
-			bsonD, err := tokensToBSON(colSpecs, tokens, uint64(0), false, false)
+			bsonD, err := tokensToBSON(fields, tokens, uint64(0))
 			So(err, ShouldBeNil)
 			So(bsonD, ShouldResemble, expectedDocument)
 		})
 		Convey("if there are more tokens than fields, additional fields should be prefixed"+
 			" with 'fields' and an index indicating the header number", func() {
-			colSpecs := []ColumnSpec{
-				{"a", new(FieldAutoParser), pgAutoCast, "auto", []string{"a"}},
-				{"b", new(FieldAutoParser), pgAutoCast, "auto", []string{"b"}},
-				{"c", new(FieldAutoParser), pgAutoCast, "auto", []string{"c"}},
-			}
+			fields := []string{"a", "b", "c"}
 			tokens := []string{"1", "2", "hello", "mongodb", "user"}
 			expectedDocument := bson.D{
-				{"a", int32(1)},
-				{"b", int32(2)},
-				{"c", "hello"},
-				{"field3", "mongodb"},
-				{"field4", "user"},
+				bson.DocElem{"a", 1},
+				bson.DocElem{"b", 2},
+				bson.DocElem{"c", "hello"},
+				bson.DocElem{"field3", "mongodb"},
+				bson.DocElem{"field4", "user"},
 			}
-			bsonD, err := tokensToBSON(colSpecs, tokens, uint64(0), false, false)
+			bsonD, err := tokensToBSON(fields, tokens, uint64(0))
 			So(err, ShouldBeNil)
 			So(bsonD, ShouldResemble, expectedDocument)
 		})
 		Convey("an error should be thrown if duplicate headers are found", func() {
-			colSpecs := []ColumnSpec{
-				{"a", new(FieldAutoParser), pgAutoCast, "auto", []string{"a"}},
-				{"b", new(FieldAutoParser), pgAutoCast, "auto", []string{"b"}},
-				{"field3", new(FieldAutoParser), pgAutoCast, "auto", []string{"field3"}},
-			}
+			fields := []string{"a", "b", "field3"}
 			tokens := []string{"1", "2", "hello", "mongodb", "user"}
-			_, err := tokensToBSON(colSpecs, tokens, uint64(0), false, false)
+			_, err := tokensToBSON(fields, tokens, uint64(0))
 			So(err, ShouldNotBeNil)
 		})
 		Convey("fields with nested values should be set appropriately", func() {
-			colSpecs := []ColumnSpec{
-				{"a", new(FieldAutoParser), pgAutoCast, "auto", []string{"a"}},
-				{"b", new(FieldAutoParser), pgAutoCast, "auto", []string{"b"}},
-				{"c.a", new(FieldAutoParser), pgAutoCast, "auto", []string{"c", "a"}},
-			}
+			fields := []string{"a", "b", "c.a"}
 			tokens := []string{"1", "2", "hello"}
-			c := bson.D{
-				{"a", "hello"},
-			}
 			expectedDocument := bson.D{
-				{"a", int32(1)},
-				{"b", int32(2)},
-				{"c", c},
+				bson.DocElem{"a", 1},
+				bson.DocElem{"b", 2},
+				bson.DocElem{"c", bson.D{
+					bson.DocElem{"a", "hello"},
+				}},
 			}
-			bsonD, err := tokensToBSON(colSpecs, tokens, uint64(0), false, false)
+			bsonD, err := tokensToBSON(fields, tokens, uint64(0))
 			So(err, ShouldBeNil)
-			So(expectedDocument[0].Key, ShouldResemble, bsonD[0].Key)
+			So(expectedDocument[0].Name, ShouldResemble, bsonD[0].Name)
 			So(expectedDocument[0].Value, ShouldResemble, bsonD[0].Value)
-			So(expectedDocument[1].Key, ShouldResemble, bsonD[1].Key)
+			So(expectedDocument[1].Name, ShouldResemble, bsonD[1].Name)
 			So(expectedDocument[1].Value, ShouldResemble, bsonD[1].Value)
-			So(expectedDocument[2].Key, ShouldResemble, bsonD[2].Key)
+			So(expectedDocument[2].Name, ShouldResemble, bsonD[2].Name)
 			So(expectedDocument[2].Value, ShouldResemble, *bsonD[2].Value.(*bson.D))
 		})
 	})
 }
 
 func TestProcessDocuments(t *testing.T) {
-	testtype.SkipUnlessTestType(t, testtype.UnitTestType)
+	testutil.VerifyTestType(t, testutil.UnitTestType)
 
 	Convey("Given an import worker", t, func() {
 		index := uint64(0)
 		csvConverters := []CSVConverter{
-			{
-				colSpecs: []ColumnSpec{
-					{"field1", new(FieldAutoParser), pgAutoCast, "auto", []string{"field1"}},
-					{"field2", new(FieldAutoParser), pgAutoCast, "auto", []string{"field2"}},
-					{"field3", new(FieldAutoParser), pgAutoCast, "auto", []string{"field3"}},
-				},
-				data:  []string{"a", "b", "c"},
-				index: index,
+			CSVConverter{
+				fields: []string{"field1", "field2", "field3"},
+				data:   []string{"a", "b", "c"},
+				index:  index,
 			},
-			{
-				colSpecs: []ColumnSpec{
-					{"field4", new(FieldAutoParser), pgAutoCast, "auto", []string{"field4"}},
-					{"field5", new(FieldAutoParser), pgAutoCast, "auto", []string{"field5"}},
-					{"field6", new(FieldAutoParser), pgAutoCast, "auto", []string{"field6"}},
-				},
-				data:  []string{"d", "e", "f"},
-				index: index,
+			CSVConverter{
+				fields: []string{"field4", "field5", "field6"},
+				data:   []string{"d", "e", "f"},
+				index:  index,
 			},
 		}
 		expectedDocuments := []bson.D{
-			{
-				{"field1", "a"},
-				{"field2", "b"},
-				{"field3", "c"},
-			}, {
-				{"field4", "d"},
-				{"field5", "e"},
-				{"field6", "f"},
+			bson.D{
+				bson.DocElem{"field1", "a"},
+				bson.DocElem{"field2", "b"},
+				bson.DocElem{"field3", "c"},
+			},
+			bson.D{
+				bson.DocElem{"field4", "d"},
+				bson.DocElem{"field5", "e"},
+				bson.DocElem{"field6", "f"},
 			},
 		}
 		Convey("processDocuments should execute the expected conversion for documents, "+
@@ -436,7 +388,7 @@ func TestProcessDocuments(t *testing.T) {
 			iw := &importWorker{
 				unprocessedDataChan:   inputChannel,
 				processedDocumentChan: outputChannel,
-				tomb:                  &tomb.Tomb{},
+				tomb: &tomb.Tomb{},
 			}
 			inputChannel <- csvConverters[0]
 			inputChannel <- csvConverters[1]
@@ -458,7 +410,7 @@ func TestProcessDocuments(t *testing.T) {
 			iw := &importWorker{
 				unprocessedDataChan:   inputChannel,
 				processedDocumentChan: outputChannel,
-				tomb:                  &tomb.Tomb{},
+				tomb: &tomb.Tomb{},
 			}
 			inputChannel <- csvConverters[0]
 			inputChannel <- csvConverters[1]
@@ -477,7 +429,7 @@ func TestProcessDocuments(t *testing.T) {
 }
 
 func TestDoSequentialStreaming(t *testing.T) {
-	testtype.SkipUnlessTestType(t, testtype.UnitTestType)
+	testutil.VerifyTestType(t, testutil.UnitTestType)
 
 	Convey("Given some import workers, a Converters input channel and an bson.D output channel", t, func() {
 		inputChannel := make(chan Converter, 5)
@@ -491,15 +443,15 @@ func TestDoSequentialStreaming(t *testing.T) {
 			make(chan bson.D),
 		}
 		importWorkers := []*importWorker{
-			{
+			&importWorker{
 				unprocessedDataChan:   workerInputChannel[0],
 				processedDocumentChan: workerOutputChannel[0],
-				tomb:                  &tomb.Tomb{},
+				tomb: &tomb.Tomb{},
 			},
-			{
+			&importWorker{
 				unprocessedDataChan:   workerInputChannel[1],
 				processedDocumentChan: workerOutputChannel[1],
-				tomb:                  &tomb.Tomb{},
+				tomb: &tomb.Tomb{},
 			},
 		}
 		Convey("documents moving through the input channel should be processed and returned in sequence", func() {
@@ -521,7 +473,7 @@ func TestDoSequentialStreaming(t *testing.T) {
 }
 
 func TestStreamDocuments(t *testing.T) {
-	testtype.SkipUnlessTestType(t, testtype.UnitTestType)
+	testutil.VerifyTestType(t, testutil.UnitTestType)
 	Convey(`Given:
 			1. a boolean indicating streaming order
 			2. an input channel where documents are streamed in
@@ -546,12 +498,9 @@ func TestStreamDocuments(t *testing.T) {
 		Convey("the entire pipeline should complete with error if an error is encountered", func() {
 			// stream in some documents - create duplicate headers to simulate an error
 			csvConverter := CSVConverter{
-				colSpecs: []ColumnSpec{
-					{"field1", new(FieldAutoParser), pgAutoCast, "auto", []string{"field1"}},
-					{"field2", new(FieldAutoParser), pgAutoCast, "auto", []string{"field2"}},
-				},
-				data:  []string{"a", "b", "c"},
-				index: uint64(0),
+				fields: []string{"field1", "field2"},
+				data:   []string{"a", "b", "c"},
+				index:  uint64(0),
 			}
 			inputChannel <- csvConverter
 			close(inputChannel)
@@ -563,7 +512,7 @@ func TestStreamDocuments(t *testing.T) {
 }
 
 func TestChannelQuorumError(t *testing.T) {
-	testtype.SkipUnlessTestType(t, testtype.UnitTestType)
+	testutil.VerifyTestType(t, testutil.UnitTestType)
 	Convey("Given a channel and a quorum...", t, func() {
 		Convey("an error should be returned if one is received", func() {
 			ch := make(chan error, 2)
@@ -586,3 +535,30 @@ func TestChannelQuorumError(t *testing.T) {
 		})
 	})
 }
+
+func TestFilterIngestError(t *testing.T) {
+	testutil.VerifyTestType(t, testutil.UnitTestType)
+
+	Convey("Given a boolean 'stopOnError' and an error...", t, func() {
+
+		Convey("an error should be returned if stopOnError is true the err is not nil", func() {
+			So(filterIngestError(true, fmt.Errorf("")), ShouldNotBeNil)
+		})
+
+		Convey("errLostConnection should be returned if stopOnError is true the err is io.EOF", func() {
+			So(filterIngestError(true, io.EOF).Error(), ShouldEqual, db.ErrLostConnection)
+		})
+
+		Convey("no error should be returned if stopOnError is false the err is not nil", func() {
+			So(filterIngestError(false, fmt.Errorf("")), ShouldBeNil)
+		})
+
+		Convey("no error should be returned if stopOnError is false the err is nil", func() {
+			So(filterIngestError(false, nil), ShouldBeNil)
+		})
+
+		Convey("no error should be returned if stopOnError is true the err is nil", func() {
+			So(filterIngestError(true, nil), ShouldBeNil)
+		})
+	})
+}
