@@ -1,46 +1,45 @@
-/**
- *    Copyright (C) 2018-present MongoDB, Inc.
+// @file bsonobj.h
+
+/*    Copyright 2009 10gen Inc.
  *
- *    This program is free software: you can redistribute it and/or modify
- *    it under the terms of the Server Side Public License, version 1,
- *    as published by MongoDB, Inc.
+ *    This program is free software: you can redistribute it and/or  modify
+ *    it under the terms of the GNU Affero General Public License, version 3,
+ *    as published by the Free Software Foundation.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    Server Side Public License for more details.
+ *    GNU Affero General Public License for more details.
  *
- *    You should have received a copy of the Server Side Public License
- *    along with this program. If not, see
- *    <http://www.mongodb.com/licensing/server-side-public-license>.
+ *    You should have received a copy of the GNU Affero General Public License
+ *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the Server Side Public License in all respects for
- *    all of the code used other than as permitted herein. If you modify file(s)
- *    with this exception, you may extend this exception to your version of the
- *    file(s), but you are not obligated to do so. If you do not wish to do so,
- *    delete this exception statement from your version. If you delete this
- *    exception statement from all source files in the program, then also delete
- *    it in the license file.
+ *    must comply with the GNU Affero General Public License in all respects
+ *    for all of the code used other than as permitted herein. If you modify
+ *    file(s) with this exception, you may extend this exception to your
+ *    version of the file(s), but you are not obligated to do so. If you do not
+ *    wish to do so, delete this exception statement from your version. If you
+ *    delete this exception statement from all source files in the program,
+ *    then also delete it in the license file.
  */
 
 #pragma once
 
 #include <bitset>
+#include <boost/container/flat_set.hpp>
 #include <list>
 #include <set>
 #include <string>
-#include <type_traits>
 #include <utility>
 #include <vector>
 
 #include "mongo/base/data_type.h"
+#include "mongo/base/disallow_copying.h"
 #include "mongo/base/string_data.h"
-#include "mongo/base/string_data_comparator_interface.h"
-#include "mongo/bson/bson_comparator_interface_base.h"
 #include "mongo/bson/bsonelement.h"
 #include "mongo/bson/bsontypes.h"
 #include "mongo/bson/oid.h"
@@ -52,11 +51,9 @@
 
 namespace mongo {
 
-class BSONObjBuilder;
-class BSONObjStlIterator;
-class ExtendedCanonicalV200Generator;
-class ExtendedRelaxedV200Generator;
-class LegacyStrictGenerator;
+typedef std::set<BSONElement, BSONElementCmpWithoutField> BSONElementSet;
+typedef boost::container::flat_set<BSONElement, BSONElementCmpWithoutField> BSONElementFlatSet;
+typedef std::multiset<BSONElement, BSONElementCmpWithoutField> BSONElementMSet;
 
 /**
    C++ representation of a "BSON" object -- that is, an extended JSON-style
@@ -99,28 +96,6 @@ class LegacyStrictGenerator;
  */
 class BSONObj {
 public:
-    struct DefaultSizeTrait {
-        constexpr static int MaxSize = BSONObjMaxInternalSize;
-    };
-    struct LargeSizeTrait {
-        constexpr static int MaxSize = BufferMaxSize;
-    };
-
-    // Declared in bsonobj_comparator_interface.h.
-    class ComparatorInterface;
-
-    /**
-     * Operator overloads for relops return a DeferredComparison which can subsequently be evaluated
-     * by a BSONObj::ComparatorInterface.
-     */
-    using DeferredComparison = BSONComparatorInterfaceBase<BSONObj>::DeferredComparison;
-
-    /**
-     * Set of rules that dictate the behavior of the comparison APIs.
-     */
-    using ComparisonRules = BSONComparatorInterfaceBase<BSONObj>::ComparisonRules;
-    using ComparisonRulesSet = BSONComparatorInterfaceBase<BSONObj>::ComparisonRulesSet;
-
     static const char kMinBSONLength = 5;
 
     /** Construct an empty BSONObj -- that is, {}. */
@@ -135,18 +110,17 @@ public:
 
     /** Construct a BSONObj from data in the proper format.
      *  Use this constructor when something else owns bsonData's buffer
-     */
-    template <typename Traits = DefaultSizeTrait>
-    explicit BSONObj(const char* bsonData, Traits t = Traits{}) {
-        init<Traits>(bsonData);
+    */
+    explicit BSONObj(const char* bsonData) {
+        init(bsonData);
     }
 
-    explicit BSONObj(ConstSharedBuffer ownedBuffer)
+    explicit BSONObj(SharedBuffer ownedBuffer)
         : _objdata(ownedBuffer.get() ? ownedBuffer.get() : BSONObj().objdata()),
           _ownedBuffer(std::move(ownedBuffer)) {}
 
     /** Move construct a BSONObj */
-    BSONObj(BSONObj&& other) noexcept
+    BSONObj(BSONObj&& other)
         : _objdata(std::move(other._objdata)), _ownedBuffer(std::move(other._ownedBuffer)) {
         other._objdata = BSONObj()._objdata;  // To return to an empty state.
         dassert(!other.isOwned());
@@ -161,13 +135,13 @@ public:
     /** Provide assignment semantics. We use the value taking form so that we can use copy
      *  and swap, and consume both lvalue and rvalue references.
      */
-    BSONObj& operator=(BSONObj otherCopy) noexcept {
+    BSONObj& operator=(BSONObj otherCopy) {
         this->swap(otherCopy);
         return *this;
     }
 
     /** Swap this BSONObj with 'other' */
-    void swap(BSONObj& other) noexcept {
+    void swap(BSONObj& other) {
         using std::swap;
         swap(_objdata, other._objdata);
         swap(_ownedBuffer, other._ownedBuffer);
@@ -202,110 +176,35 @@ public:
        @return true if this is in owned mode
     */
     bool isOwned() const {
-        return bool(_ownedBuffer);
+        return _ownedBuffer.get() != 0;
     }
 
-    /**
-     * Share ownership with another object.
-     *
-     * It is the callers responsibility to ensure that the other object is owned and contains the
-     * data this BSONObj is viewing. This can happen if this is a subobject or sibling object
-     * contained in a larger buffer.
-     */
-    BSONObj& shareOwnershipWith(ConstSharedBuffer buffer) & {
-        invariant(buffer);
-        _ownedBuffer = buffer;
-        return *this;
-    }
-    BSONObj& shareOwnershipWith(const BSONObj& other) & {
-        shareOwnershipWith(other.sharedBuffer());
-        return *this;
-    }
-    BSONObj&& shareOwnershipWith(ConstSharedBuffer buffer) && {
-        return std::move(shareOwnershipWith(buffer));
-    }
-    BSONObj&& shareOwnershipWith(const BSONObj& other) && {
-        return std::move(shareOwnershipWith(other));
-    }
-
-    const ConstSharedBuffer& sharedBuffer() const {
-        invariant(isOwned());
-        return _ownedBuffer;
-    }
-
-    ConstSharedBuffer releaseSharedBuffer() {
-        invariant(isOwned());
-        BSONObj sink = std::move(*this);  // Leave *this in a valid moved-from state.
-        return std::move(sink._ownedBuffer);
-    }
-
-    /** If the data buffer is under the control of this BSONObj, return it.
-        Else return an owned copy.
+    /** assure the data buffer is under the control of this BSONObj and not a remote buffer
+        @see isOwned()
     */
     BSONObj getOwned() const;
 
-    /** Returns an owned copy of the given BSON object. */
-    static BSONObj getOwned(const BSONObj& obj);
-
     /** @return a new full (and owned) copy of the object. */
     BSONObj copy() const;
 
-    /** @return a new full (and owned) redacted copy of the object. */
-    BSONObj redact() const;
-
     /** Readable representation of a BSON object in an extended JSON-style notation.
         This is an abbreviated representation which might be used for logging.
     */
     enum { maxToStringRecursionDepth = 100 };
 
-    std::string toString(bool redactValues = false) const;
-    void toString(StringBuilder& s,
-                  bool isArray = false,
-                  bool full = false,
-                  bool redactValues = false,
-                  int depth = 0) const;
+    std::string toString(bool isArray = false, bool full = false) const;
+    void toString(StringBuilder& s, bool isArray = false, bool full = false, int depth = 0) const;
 
     /** Properly formatted JSON string.
         @param pretty if true we try to add some lf's and indentation
     */
-    std::string jsonString(JsonStringFormat format = ExtendedCanonicalV2_0_0,
+    std::string jsonString(JsonStringFormat format = Strict,
                            int pretty = 0,
-                           bool isArray = false,
-                           size_t writeLimit = 0,
-                           BSONObj* outTruncationResult = nullptr) const;
-
-    BSONObj jsonStringBuffer(JsonStringFormat format,
-                             int pretty,
-                             bool isArray,
-                             fmt::memory_buffer& buffer,
-                             size_t writeLimit = 0) const;
-
-    BSONObj jsonStringGenerator(ExtendedCanonicalV200Generator const& generator,
-                                int pretty,
-                                bool isArray,
-                                fmt::memory_buffer& buffer,
-                                size_t writeLimit = 0) const;
-    BSONObj jsonStringGenerator(ExtendedRelaxedV200Generator const& generator,
-                                int pretty,
-                                bool isArray,
-                                fmt::memory_buffer& buffer,
-                                size_t writeLimit = 0) const;
-    BSONObj jsonStringGenerator(LegacyStrictGenerator const& generator,
-                                int pretty,
-                                bool isArray,
-                                fmt::memory_buffer& buffer,
-                                size_t writeLimit = 0) const;
+                           bool isArray = false) const;
 
     /** note: addFields always adds _id even if not specified */
     int addFields(BSONObj& from, std::set<std::string>& fields); /* returns n added */
 
-    /**
-     * Add specific field to the end of the object if it did not exist, otherwise replace it
-     * preserving original field order. Returns newly built object. Returns copy of this for empty
-     * field.
-     */
-    BSONObj addField(const BSONElement& field) const;
-
     /** remove specified field and return a new object with the remaining fields.
         slowish as builds a full new object
      */
@@ -316,11 +215,26 @@ public:
     */
     int nFields() const;
 
-    /**
-     * Returns a 'Container' populated with the field names of the object.
+    /** adds the field names to the fields set.  does NOT clear it (appends). */
+    int getFieldNames(std::set<std::string>& fields) const;
+
+    /** @return the specified element.  element.eoo() will be true if not found.
+        @param name field to find. supports dot (".") notation to reach into embedded objects.
+         for example "x.y" means "in the nested object in field x, retrieve field y"
+    */
+    BSONElement getFieldDotted(StringData name) const;
+
+    /** Like getFieldDotted(), but expands arrays and returns all matching objects.
+     *  Turning off expandLastArray allows you to retrieve nested array objects instead of
+     *  their contents.
      */
-    template <class Container>
-    Container getFieldNames() const;
+    void getFieldsDotted(StringData name, BSONElementSet& ret, bool expandLastArray = true) const;
+    void getFieldsDotted(StringData name, BSONElementMSet& ret, bool expandLastArray = true) const;
+
+    /** Like getFieldDotted(), but returns first array encountered while traversing the
+        dotted fields of name.  The name variable is updated to represent field
+        names with respect to the returned element. */
+    BSONElement getFieldDottedOrArray(const char*& name) const;
 
     /** Get the field of the specified name. eoo() is true on the returned
         element if not found.
@@ -393,12 +307,17 @@ public:
      *    this.extractFieldsUnDotted({a : 1 , c : 1}) -> {"" : 4 , "" : 6 }
      *    this.extractFieldsUnDotted({b : "blah"}) -> {"" : 5}
      *
-     */
-    BSONObj extractFieldsUndotted(const BSONObj& pattern) const;
-    void extractFieldsUndotted(BSONObjBuilder* b, const BSONObj& pattern) const;
+    */
+    BSONObj extractFieldsUnDotted(const BSONObj& pattern) const;
+
+    /** extract items from object which match a pattern object.
+        e.g., if pattern is { x : 1, y : 1 }, builds an object with
+        x and y elements of this object, if they are present.
+       returns elements with original field names
+    */
+    BSONObj extractFields(const BSONObj& pattern, bool fillWithNull = false) const;
 
     BSONObj filterFieldsUndotted(const BSONObj& filter, bool inFilter) const;
-    void filterFieldsUndotted(BSONObjBuilder* b, const BSONObj& filter, bool inFilter) const;
 
     BSONElement getFieldUsingIndexNames(StringData fieldName, const BSONObj& indexKey) const;
 
@@ -418,87 +337,117 @@ public:
     }
 
     /** performs a cursory check on the object's size only. */
-    template <typename Traits = DefaultSizeTrait>
     bool isValid() const {
-        static_assert(Traits::MaxSize > 0 && Traits::MaxSize <= std::numeric_limits<int>::max(),
-                      "BSONObj maximum size must be within possible limits");
         int x = objsize();
-        return x > 0 && x <= Traits::MaxSize;
+        return x > 0 && x <= BSONObjMaxInternalSize;
+    }
+
+    /** @return ok if it can be stored as a valid embedded doc.
+     *  Not valid if any field name:
+     *      - contains a "."
+     *      - starts with "$"
+     *          -- unless it is a dbref ($ref/$id/[$db]/...)
+     */
+    inline bool okForStorage() const {
+        return _okForStorage(false, true).isOK();
+    }
+
+    /** Same as above with the following extra restrictions
+     *  Not valid if:
+     *      - "_id" field is a
+     *          -- Regex
+     *          -- Array
+     */
+    inline bool okForStorageAsRoot() const {
+        return _okForStorage(true, true).isOK();
     }
 
     /**
-     * Validates that the element is okay to be stored in a collection.
-     * Recursively validates children.
+     * Validates that this can be stored as an embedded document
+     * See details above in okForStorage
+     *
+     * If 'deep' is true then validation is done to children
+     *
+     * If not valid a user readable status message is returned.
      */
-    Status storageValidEmbedded() const;
+    inline Status storageValidEmbedded(const bool deep = true) const {
+        return _okForStorage(false, deep);
+    }
+
+    /**
+     * Validates that this can be stored as a document (in a collection)
+     * See details above in okForStorageAsRoot
+     *
+     * If 'deep' is true then validation is done to children
+     *
+     * If not valid a user readable status message is returned.
+     */
+    inline Status storageValid(const bool deep = true) const {
+        return _okForStorage(true, deep);
+    }
 
     /** @return true if object is empty -- i.e.,  {} */
     bool isEmpty() const {
         return objsize() <= kMinBSONLength;
     }
 
+    void dump() const;
+
     /** Alternative output format */
     std::string hexDump() const;
 
-    //
-    // Comparison API.
-    //
-    // BSONObj instances can be compared either using woCompare() or via operator overloads. Most
-    // callers should prefer operator overloads. Note that the operator overloads return a
-    // DeferredComparison, which must be subsequently evaluated by a BSONObj::ComparatorInterface.
-    // See bsonobj_comparator_interface.h for details.
-    //
-
-    /**
-     * Compares two BSON Objects using the rules specified by 'rules', 'comparator' for
-     * string comparisons, and 'o' for ascending vs. descending ordering.
-     *
-     * Returns <0 if 'this' is less than 'obj'.
-     *         >0 if 'this' is greater than 'obj'.
-     *          0 if 'this' is equal to 'obj'.
-     */
-    int woCompare(const BSONObj& r,
-                  const Ordering& o,
-                  ComparisonRulesSet rules = ComparisonRules::kConsiderFieldName,
-                  const StringData::ComparatorInterface* comparator = nullptr) const;
+    /**wo='well ordered'.  fields must be in same order in each object.
+       Ordering is with respect to the signs of the elements
+       and allows ascending / descending key mixing.
+       @return  <0 if l<r. 0 if l==r. >0 if l>r
+    */
+    int woCompare(const BSONObj& r, const Ordering& o, bool considerFieldName = true) const;
 
+    /**wo='well ordered'.  fields must be in same order in each object.
+       Ordering is with respect to the signs of the elements
+       and allows ascending / descending key mixing.
+       @return  <0 if l<r. 0 if l==r. >0 if l>r
+    */
     int woCompare(const BSONObj& r,
                   const BSONObj& ordering = BSONObj(),
-                  ComparisonRulesSet rules = ComparisonRules::kConsiderFieldName,
-                  const StringData::ComparatorInterface* comparator = nullptr) const;
+                  bool considerFieldName = true) const;
 
-    DeferredComparison operator<(const BSONObj& other) const {
-        return DeferredComparison(DeferredComparison::Type::kLT, *this, other);
+    bool operator<(const BSONObj& other) const {
+        return woCompare(other) < 0;
     }
-
-    DeferredComparison operator<=(const BSONObj& other) const {
-        return DeferredComparison(DeferredComparison::Type::kLTE, *this, other);
+    bool operator<=(const BSONObj& other) const {
+        return woCompare(other) <= 0;
     }
-
-    DeferredComparison operator>(const BSONObj& other) const {
-        return DeferredComparison(DeferredComparison::Type::kGT, *this, other);
+    bool operator>(const BSONObj& other) const {
+        return woCompare(other) > 0;
     }
-
-    DeferredComparison operator>=(const BSONObj& other) const {
-        return DeferredComparison(DeferredComparison::Type::kGTE, *this, other);
-    }
-
-    DeferredComparison operator==(const BSONObj& other) const {
-        return DeferredComparison(DeferredComparison::Type::kEQ, *this, other);
-    }
-
-    DeferredComparison operator!=(const BSONObj& other) const {
-        return DeferredComparison(DeferredComparison::Type::kNE, *this, other);
+    bool operator>=(const BSONObj& other) const {
+        return woCompare(other) >= 0;
     }
 
     /**
-     * Returns true if 'this' is a prefix of otherObj- in other words if otherObj contains the same
-     * field names and field vals in the same order as 'this', plus optionally some additional
-     * elements.
-     *
-     * All comparisons between elements are made using 'eltCmp'.
+     * @param useDotted whether to treat sort key fields as possibly dotted and expand into them
      */
-    bool isPrefixOf(const BSONObj& otherObj, const BSONElement::ComparatorInterface& eltCmp) const;
+    int woSortOrder(const BSONObj& r, const BSONObj& sortKey, bool useDotted = false) const;
+
+    bool equal(const BSONObj& r) const;
+
+    /**
+     * Functor compatible with std::hash for std::unordered_{map,set}
+     * Warning: The hash function is subject to change. Do not use in cases where hashes need
+     *          to be consistent across versions.
+     */
+    struct Hasher {
+        size_t operator()(const BSONObj& obj) const;
+    };
+
+    /**
+     * @param otherObj
+     * @return true if 'this' is a prefix of otherObj- in other words if
+     * otherObj contains the same field names and field vals in the same
+     * order as 'this', plus optionally some additional elements.
+     */
+    bool isPrefixOf(const BSONObj& otherObj) const;
 
     /**
      * @param otherObj
@@ -532,10 +481,6 @@ public:
         return *p == EOO ? "" : p + 1;
     }
 
-    StringData firstElementFieldNameStringData() const {
-        return StringData(firstElementFieldName());
-    }
-
     BSONType firstElementType() const {
         const char* p = objdata() + 4;
         return (BSONType)*p;
@@ -558,15 +503,41 @@ public:
         passed object. */
     BSONObj replaceFieldNames(const BSONObj& obj) const;
 
-    static BSONObj stripFieldNames(const BSONObj& obj);
+    /** true unless corrupt */
+    bool valid() const;
 
-    bool hasFieldNames() const;
+    bool operator==(const BSONObj& other) const {
+        return equal(other);
+    }
+    bool operator!=(const BSONObj& other) const {
+        return !operator==(other);
+    }
 
-    /**
-     * Returns true if this object is valid according to the specified BSON version, and returns
-     * false otherwise.
-     */
-    bool valid(BSONVersion version) const;
+    enum MatchType {
+        Equality = 0,
+        LT = 0x1,
+        LTE = 0x3,
+        GTE = 0x6,
+        GT = 0x4,
+        opIN = 0x8,  // { x : { $in : [1,2,3] } }
+        NE = 0x9,
+        opSIZE = 0x0A,
+        opALL = 0x0B,
+        NIN = 0x0C,
+        opEXISTS = 0x0D,
+        opMOD = 0x0E,
+        opTYPE = 0x0F,
+        opREGEX = 0x10,
+        opOPTIONS = 0x11,
+        opELEM_MATCH = 0x12,
+        opNEAR = 0x13,
+        opWITHIN = 0x14,
+        opGEO_INTERSECTS = 0x16,
+        opBITS_ALL_SET = 0x17,
+        opBITS_ALL_CLEAR = 0x18,
+        opBITS_ANY_SET = 0x19,
+        opBITS_ANY_CLEAR = 0x1A,
+    };
 
     /** add all elements of the object to the specified vector */
     void elems(std::vector<BSONElement>&) const;
@@ -574,9 +545,7 @@ public:
     void elems(std::list<BSONElement>&) const;
 
     friend class BSONObjIterator;
-    friend class BSONObjStlIterator;
-    typedef BSONObjStlIterator iterator;
-    typedef BSONObjStlIterator const_iterator;
+    typedef BSONObjIterator iterator;
 
     /**
      * These enable range-based for loops over BSONObjs:
@@ -585,8 +554,8 @@ public:
      *          ... // Do something with elem
      *      }
      */
-    iterator begin() const;
-    iterator end() const;
+    BSONObjIterator begin() const;
+    BSONObjIterator end() const;
 
     void appendSelfToBufBuilder(BufBuilder& b) const {
         verify(objsize());
@@ -596,13 +565,26 @@ public:
     template <typename T>
     bool coerceVector(std::vector<T>* out) const;
 
+    typedef SharedBuffer::Holder Holder;
+
+    /** Given a pointer to a region of un-owned memory containing BSON data, prefixed by
+     *  sufficient space for a BSONObj::Holder object, return a BSONObj that owns the
+     *  memory.
+     *
+     * This class will call free(holderPrefixedData), so it must have been allocated in a way
+     * that makes that valid.
+     */
+    static BSONObj takeOwnership(char* holderPrefixedData) {
+        return BSONObj(SharedBuffer::takeOwnership(holderPrefixedData));
+    }
+
     /// members for Sorter
     struct SorterDeserializeSettings {};  // unused
     void serializeForSorter(BufBuilder& buf) const {
         buf.appendBuf(objdata(), objsize());
     }
     static BSONObj deserializeForSorter(BufReader& buf, const SorterDeserializeSettings&) {
-        const int size = buf.peek<LittleEndian<int>>();
+        const int size = buf.peek<int>();
         const void* ptr = buf.skip(size);
         return BSONObj(static_cast<const char*>(ptr));
     }
@@ -612,38 +594,33 @@ public:
     }
 
 private:
-    template <typename Generator>
-    BSONObj _jsonStringGenerator(const Generator& g,
-                                 int pretty,
-                                 bool isArray,
-                                 fmt::memory_buffer& buffer,
-                                 size_t writeLimit) const;
+    void _assertInvalid() const;
 
-    void _assertInvalid(int maxSize) const;
-
-    template <typename Traits = DefaultSizeTrait>
     void init(const char* data) {
         _objdata = data;
-        if (!isValid<Traits>())
-            _assertInvalid(Traits::MaxSize);
+        if (!isValid())
+            _assertInvalid();
     }
 
-    void _validateUnownedSize(int size) const;
+    /**
+     * Validate if the element is okay to be stored in a collection, maybe as the root element
+     *
+     * If 'root' is true then checks against _id are made.
+     * If 'deep' is false then do not traverse through children
+     */
+    Status _okForStorage(bool root, bool deep) const;
 
     const char* _objdata;
-    ConstSharedBuffer _ownedBuffer;
+    SharedBuffer _ownedBuffer;
 };
 
-MONGO_STATIC_ASSERT(std::is_nothrow_move_constructible_v<BSONObj>);
-MONGO_STATIC_ASSERT(std::is_nothrow_move_assignable_v<BSONObj>);
-
 std::ostream& operator<<(std::ostream& s, const BSONObj& o);
 std::ostream& operator<<(std::ostream& s, const BSONElement& e);
 
 StringBuilder& operator<<(StringBuilder& s, const BSONObj& o);
 StringBuilder& operator<<(StringBuilder& s, const BSONElement& e);
 
-inline void swap(BSONObj& l, BSONObj& r) noexcept {
+inline void swap(BSONObj& l, BSONObj& r) {
     l.swap(r);
 }
 
@@ -653,93 +630,26 @@ struct BSONArray : BSONObj {
     explicit BSONArray(const BSONObj& obj) : BSONObj(obj) {}
 };
 
-/**
- * An stl-compatible forward iterator over the elements of a BSONObj.
- *
- * The BSONObj must stay in scope for the duration of the iterator's execution.
- */
-class BSONObjStlIterator {
-public:
-    using iterator_category = std::forward_iterator_tag;
-    using difference_type = ptrdiff_t;
-    using value_type = BSONElement;
-    using pointer = const BSONElement*;
-    using reference = const BSONElement&;
+/** iterator for a BSONObj
 
-    /**
-     * All default constructed iterators are equal to each other.
-     * They are in a dereferencable state, and return an EOO BSONElement.
-     * They must not be incremented.
-     */
-    BSONObjStlIterator() = default;
+   Note each BSONObj ends with an EOO element: so you will get more() on an empty
+   object, although next().eoo() will be true.
 
-    /**
-     * Constructs an iterator pointing to the first element in obj or EOO if it is empty.
-     */
-    explicit BSONObjStlIterator(const BSONObj& obj) : BSONObjStlIterator(obj.firstElement()) {}
+   The BSONObj must stay in scope for the duration of the iterator's execution.
 
-    /**
-     * Returns an iterator pointing to the EOO element in obj.
-     */
-    static BSONObjStlIterator endOf(const BSONObj& obj) {
-        auto eooElem = BSONElement();
-        eooElem.data = (obj.objdata() + obj.objsize() - 1);
-        dassert(eooElem.eoo());  // This is checked in the BSONObj constructor.
-        return BSONObjStlIterator(eooElem);
-    }
-
-    /** pre-increment */
-    BSONObjStlIterator& operator++() {
-        dassert(!_cur.eoo());
-        *this = BSONObjStlIterator(BSONElement(_cur.rawdata() + _cur.size()));
-        return *this;
-    }
-
-    /** post-increment */
-    BSONObjStlIterator operator++(int) {
-        BSONObjStlIterator oldPos = *this;
-        ++*this;
-        return oldPos;
-    }
-
-    const BSONElement& operator*() const {
-        return _cur;
-    }
-    const BSONElement* operator->() const {
-        return &_cur;
-    }
-
-    bool operator==(const BSONObjStlIterator& other) {
-        return _cur.rawdata() == other._cur.rawdata();
-    }
-    bool operator!=(const BSONObjStlIterator& other) {
-        return !(*this == other);
-    }
-
-private:
-    explicit BSONObjStlIterator(BSONElement elem) : _cur(elem) {}
-
-    BSONElement _cur;
-};
-
-/**
- * Non-STL iterator for a BSONObj
- *
- * For simple loops over BSONObj, do this instead: for (auto&& elem : obj) { ... }
- *
- * Note each BSONObj ends with an EOO element: so you will get moreWithEOO() on an empty
- * object, although more() will be false and next().eoo() will be true.
- *
- * The BSONObj must stay in scope for the duration of the iterator's execution.
- */
+   todo: Finish making this an STL-compatible iterator.
+            Need iterator_catagory et al (maybe inherit from std::iterator).
+            Need operator->
+            operator* should return a const reference not a value.
+*/
 class BSONObjIterator {
 public:
     /** Create an iterator for a BSON object.
-     */
+    */
     explicit BSONObjIterator(const BSONObj& jso) {
         int sz = jso.objsize();
         if (MONGO_unlikely(sz == 0)) {
-            _pos = _theend = nullptr;
+            _pos = _theend = 0;
             return;
         }
         _pos = jso.objdata() + 4;
@@ -751,17 +661,43 @@ public:
         _theend = end - 1;
     }
 
+    static BSONObjIterator endOf(const BSONObj& obj) {
+        BSONObjIterator end(obj);
+        end._pos = end._theend;
+        return end;
+    }
+
     /** @return true if more elements exist to be enumerated. */
-    bool more() const {
+    bool more() {
         return _pos < _theend;
     }
 
     /** @return true if more elements exist to be enumerated INCLUDING the EOO element which is
      * always at the end. */
-    bool moreWithEOO() const {
+    bool moreWithEOO() {
         return _pos <= _theend;
     }
 
+    /**
+     * @return the next element in the object. For the final element, element.eoo() will be true.
+     */
+    BSONElement next(bool checkEnd) {
+        verify(_pos <= _theend);
+
+        int maxLen = -1;
+        if (checkEnd) {
+            maxLen = _theend + 1 - _pos;
+            verify(maxLen > 0);
+        }
+
+        BSONElement e(_pos, maxLen);
+        int esize = e.size(maxLen);
+        massert(16446, "BSONElement has bad size", esize > 0);
+        _pos += esize;
+
+        return e;
+    }
+
     BSONElement next() {
         verify(_pos <= _theend);
         BSONElement e(_pos);
@@ -803,8 +739,7 @@ private:
 
 /** Base class implementing ordered iteration through BSONElements. */
 class BSONIteratorSorted {
-    BSONIteratorSorted(const BSONIteratorSorted&) = delete;
-    BSONIteratorSorted& operator=(const BSONIteratorSorted&) = delete;
+    MONGO_DISALLOW_COPYING(BSONIteratorSorted);
 
 public:
     ~BSONIteratorSorted() {
@@ -828,7 +763,7 @@ protected:
 
 private:
     const int _nfields;
-    const std::unique_ptr<const char*[]> _fields;
+    const std::unique_ptr<const char* []> _fields;
     int _cur;
 };
 
@@ -848,11 +783,11 @@ public:
     BSONArrayIteratorSorted(const BSONArray& array);
 };
 
-inline BSONObj::iterator BSONObj::begin() const {
-    return BSONObj::iterator(*this);
+inline BSONObjIterator BSONObj::begin() const {
+    return BSONObjIterator(*this);
 }
-inline BSONObj::iterator BSONObj::end() const {
-    return BSONObj::iterator::endOf(*this);
+inline BSONObjIterator BSONObj::end() const {
+    return BSONObjIterator::endOf(*this);
 }
 
 /**
@@ -868,7 +803,7 @@ struct DataType::Handler<BSONObj> {
                        const char* ptr,
                        size_t length,
                        size_t* advanced,
-                       std::ptrdiff_t debug_offset) noexcept try {
+                       std::ptrdiff_t debug_offset) {
         auto temp = BSONObj(ptr);
         auto len = temp.objsize();
         if (bson) {
@@ -878,15 +813,13 @@ struct DataType::Handler<BSONObj> {
             *advanced = len;
         }
         return Status::OK();
-    } catch (const DBException& e) {
-        return e.toStatus();
     }
 
     static Status store(const BSONObj& bson,
                         char* ptr,
                         size_t length,
                         size_t* advanced,
-                        std::ptrdiff_t debug_offset) noexcept;
+                        std::ptrdiff_t debug_offset);
 
     static BSONObj defaultConstruct() {
         return BSONObj();
@@ -897,7 +830,9 @@ template <size_t N>
 inline void BSONObj::getFields(const std::array<StringData, N>& fieldNames,
                                std::array<BSONElement, N>* fields) const {
     std::bitset<N> foundFields;
-    for (auto&& el : *this) {
+    auto iter = this->begin();
+    while (iter.more() && !foundFields.all()) {
+        auto el = iter.next();
         auto fieldName = el.fieldNameStringData();
         for (std::size_t i = 0; i < N; ++i) {
             if (!foundFields.test(i) && (fieldNames[i] == fieldName)) {
@@ -906,20 +841,6 @@ inline void BSONObj::getFields(const std::array<StringData, N>& fieldNames,
                 break;
             }
         }
-        if (foundFields.all())
-            break;
     }
 }
-
-template <class Container>
-Container BSONObj::getFieldNames() const {
-    Container fields;
-    for (auto&& elem : *this) {
-        if (elem.eoo())
-            break;
-        fields.insert(elem.fieldName());
-    }
-    return fields;
-}
-
 }  // namespace mongo
