@@ -1,24 +1,25 @@
+// cloner.h - copy a database (export/import basically)
+
 /**
- *    Copyright (C) 2018-present MongoDB, Inc.
+ *    Copyright (C) 2011 10gen Inc.
  *
- *    This program is free software: you can redistribute it and/or modify
- *    it under the terms of the Server Side Public License, version 1,
- *    as published by MongoDB, Inc.
+ *    This program is free software: you can redistribute it and/or  modify
+ *    it under the terms of the GNU Affero General Public License, version 3,
+ *    as published by the Free Software Foundation.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    Server Side Public License for more details.
+ *    GNU Affero General Public License for more details.
  *
- *    You should have received a copy of the Server Side Public License
- *    along with this program. If not, see
- *    <http://www.mongodb.com/licensing/server-side-public-license>.
+ *    You should have received a copy of the GNU Affero General Public License
+ *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the Server Side Public License in all respects for
+ *    must comply with the GNU Affero General Public License in all respects for
  *    all of the code used other than as permitted herein. If you modify file(s)
  *    with this exception, you may extend this exception to your version of the
  *    file(s), but you are not obligated to do so. If you do not wish to do so,
@@ -29,15 +30,12 @@
 
 #pragma once
 
-/**
- * copy a database (export/import basically)
- */
-
-#include <string>
 #include <vector>
+#include <string>
 
-#include "mongo/client/dbclient_base.h"
-#include "mongo/db/catalog/collection_options.h"
+#include "mongo/base/disallow_copying.h"
+#include "mongo/client/dbclientinterface.h"
+#include "mongo/s/catalog/catalog_manager.h"
 
 namespace mongo {
 
@@ -48,43 +46,35 @@ class OperationContext;
 
 
 class Cloner {
-    Cloner(const Cloner&) = delete;
-    Cloner& operator=(const Cloner&) = delete;
+    MONGO_DISALLOW_COPYING(Cloner);
 
 public:
     Cloner();
 
-    void setConnection(std::unique_ptr<DBClientBase> c) {
-        _conn = std::move(c);
+    void setConnection(DBClientBase* c) {
+        _conn.reset(c);
     }
 
     /**
      * Copies an entire database from the specified host.
      * clonedColls: when not-null, the function will return with this populated with a list of
-     *              the collections that were cloned.
+     *              the collections that were cloned.  This is for the user-facing clone command.
      * collectionsToClone: When opts.createCollections is false, this list reflects the collections
      *              that are cloned.  When opts.createCollections is true, this parameter is
      *              ignored and the collection list is fetched from the remote via _conn.
      */
-    Status copyDb(OperationContext* opCtx,
+    Status copyDb(OperationContext* txn,
                   const std::string& toDBName,
                   const std::string& masterHost,
                   const CloneOptions& opts,
                   std::set<std::string>* clonedColls,
                   std::vector<BSONObj> collectionsToClone = std::vector<BSONObj>());
 
-    /**
-     * Copies a collection. The optionsParser indicates how to parse the collection options. If
-     * 'parseForCommand' is provided, then the UUID is ignored and a new UUID is generated. If
-     * 'parseForStorage' is provided, then the UUID will be preserved and parsed out of the
-     * options.
-     */
-    bool copyCollection(OperationContext* opCtx,
+    bool copyCollection(OperationContext* txn,
                         const std::string& ns,
                         const BSONObj& query,
                         std::string& errmsg,
-                        bool copyIndexes,
-                        CollectionOptions::ParseKind optionsParser);
+                        bool copyIndexes);
 
     // Filters a database's collection list and removes collections that should not be cloned.
     // CloneOptions should be populated with a fromDB and a list of collections to ignore, which
@@ -92,41 +82,28 @@ public:
     StatusWith<std::vector<BSONObj>> filterCollectionsForClone(
         const CloneOptions& opts, const std::list<BSONObj>& initialCollections);
 
-    struct CreateCollectionParams {
-        std::string collectionName;
-        BSONObj collectionInfo;
-        BSONObj idIndexSpec;
-        bool shardedColl = false;
-    };
-
-    // Executes 'createCollection' for each collection described in 'createCollectionParams', in
-    // 'dbName'.
-    Status createCollectionsForDb(OperationContext* opCtx,
-                                  const std::vector<CreateCollectionParams>& createCollectionParams,
-                                  const std::string& dbName,
-                                  const CloneOptions& opts);
-
-    /*
-     * Returns the _id index spec from 'indexSpecs', or an empty BSONObj if none is found.
-     */
-    static BSONObj getIdIndexSpec(const std::list<BSONObj>& indexSpecs);
+    // Executes 'createCollection' for each collection specified in 'collections', in 'dbName'.
+    Status createCollectionsForDb(OperationContext* txn,
+                                  const std::vector<BSONObj>& collections,
+                                  const std::string& dbName);
 
 private:
-    void copy(OperationContext* opCtx,
+    void copy(OperationContext* txn,
               const std::string& toDBName,
               const NamespaceString& from_ns,
               const BSONObj& from_opts,
-              const BSONObj& from_id_index,
               const NamespaceString& to_ns,
+              bool masterSameProcess,
               const CloneOptions& opts,
               Query q);
 
-    void copyIndexes(OperationContext* opCtx,
+    void copyIndexes(OperationContext* txn,
                      const std::string& toDBName,
                      const NamespaceString& from_ns,
                      const BSONObj& from_opts,
-                     const std::list<BSONObj>& from_indexes,
-                     const NamespaceString& to_ns);
+                     const NamespaceString& to_ns,
+                     bool masterSameProcess,
+                     bool slaveOk);
 
     struct Fun;
     std::unique_ptr<DBClientBase> _conn;
@@ -135,19 +112,29 @@ private:
 /**
  *  slaveOk     - if true it is ok if the source of the data is !ismaster.
  *  useReplAuth - use the credentials we normally use as a replication slave for the cloning
+ *  snapshot    - use snapshot mode for copying collections.  note this should not be used
+ *                when it isn't required, as it will be slower.  for example,
+ *                repairDatabase need not use it.
+ *  checkForCatalogChange - Internal option set for clone commands initiated by a mongos that are
+ *                holding a distributed lock (such as movePrimary).  Indicates that we need to
+ *                be periodically checking to see if the catalog manager has swapped and fail
+ *                if it has so that we don't block the mongos that initiated the command.
  *  createCollections - When 'true', will fetch a list of collections from the remote and create
  *                them.  When 'false', assumes collections have already been created ahead of time.
  */
 struct CloneOptions {
     std::string fromDB;
-    std::set<std::string> shardedColls;
+    std::set<std::string> collsToIgnore;
 
     bool slaveOk = false;
     bool useReplAuth = false;
+    bool snapshot = true;
 
     bool syncData = true;
     bool syncIndexes = true;
+    bool checkForCatalogChange = false;
     bool createCollections = true;
+    CatalogManager::ConfigServerMode initialCatalogMode = CatalogManager::ConfigServerMode::NONE;
 };
 
 }  // namespace mongo
