@@ -1,34 +1,32 @@
 // mmap_v1_extent_manager.cpp
 
-
 /**
- *    Copyright (C) 2018-present MongoDB, Inc.
- *
- *    This program is free software: you can redistribute it and/or modify
- *    it under the terms of the Server Side Public License, version 1,
- *    as published by MongoDB, Inc.
- *
- *    This program is distributed in the hope that it will be useful,
- *    but WITHOUT ANY WARRANTY; without even the implied warranty of
- *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    Server Side Public License for more details.
- *
- *    You should have received a copy of the Server Side Public License
- *    along with this program. If not, see
- *    <http://www.mongodb.com/licensing/server-side-public-license>.
- *
- *    As a special exception, the copyright holders give permission to link the
- *    code of portions of this program with the OpenSSL library under certain
- *    conditions as described in each individual source file and distribute
- *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the Server Side Public License in all respects for
- *    all of the code used other than as permitted herein. If you modify file(s)
- *    with this exception, you may extend this exception to your version of the
- *    file(s), but you are not obligated to do so. If you do not wish to do so,
- *    delete this exception statement from your version. If you delete this
- *    exception statement from all source files in the program, then also delete
- *    it in the license file.
- */
+*    Copyright (C) 2014 MongoDB Inc.
+*
+*    This program is free software: you can redistribute it and/or  modify
+*    it under the terms of the GNU Affero General Public License, version 3,
+*    as published by the Free Software Foundation.
+*
+*    This program is distributed in the hope that it will be useful,
+*    but WITHOUT ANY WARRANTY; without even the implied warranty of
+*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+*    GNU Affero General Public License for more details.
+*
+*    You should have received a copy of the GNU Affero General Public License
+*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*
+*    As a special exception, the copyright holders give permission to link the
+*    code of portions of this program with the OpenSSL library under certain
+*    conditions as described in each individual source file and distribute
+*    linked combinations including the program with the OpenSSL library. You
+*    must comply with the GNU Affero General Public License in all respects for
+*    all of the code used other than as permitted herein. If you modify file(s)
+*    with this exception, you may extend this exception to your version of the
+*    file(s), but you are not obligated to do so. If you do not wish to do so,
+*    delete this exception statement from your version. If you delete this
+*    exception statement from all source files in the program, then also delete
+*    it in the license file.
+*/
 
 #define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kStorage
 
@@ -39,18 +37,17 @@
 #include "mongo/base/counter.h"
 #include "mongo/db/audit.h"
 #include "mongo/db/client.h"
-#include "mongo/db/concurrency/d_concurrency.h"
-#include "mongo/db/operation_context.h"
 #include "mongo/db/service_context.h"
-#include "mongo/db/storage/mmap_v1/data_file.h"
 #include "mongo/db/storage/mmap_v1/dur.h"
+#include "mongo/db/storage/mmap_v1/data_file.h"
+#include "mongo/db/storage/mmap_v1/record.h"
 #include "mongo/db/storage/mmap_v1/extent.h"
 #include "mongo/db/storage/mmap_v1/extent_manager.h"
 #include "mongo/db/storage/mmap_v1/mmap.h"
 #include "mongo/db/storage/mmap_v1/mmap_v1_engine.h"
 #include "mongo/db/storage/mmap_v1/mmap_v1_options.h"
-#include "mongo/db/storage/mmap_v1/record.h"
 #include "mongo/db/storage/record_fetcher.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/stdx/memory.h"
 #include "mongo/util/fail_point_service.h"
 #include "mongo/util/file.h"
@@ -69,12 +66,11 @@ using std::stringstream;
 // recordNeedsFetch().
 static const int kNeedsFetchFailFreq = 2;
 static Counter64 needsFetchFailCounter;
-MONGO_FAIL_POINT_DEFINE(recordNeedsFetchFail);
+MONGO_FP_DECLARE(recordNeedsFetchFail);
 
 // Used to make sure the compiler doesn't get too smart on us when we're
 // trying to touch records.
-// volatile - avoid compiler optimizations for touching a mmap page
-volatile int __record_touch_dummy = 1;  // NOLINT
+volatile int __record_touch_dummy = 1;
 
 class MmapV1RecordFetcher : public RecordFetcher {
     MONGO_DISALLOW_COPYING(MmapV1RecordFetcher);
@@ -82,9 +78,9 @@ class MmapV1RecordFetcher : public RecordFetcher {
 public:
     explicit MmapV1RecordFetcher(const MmapV1RecordHeader* record) : _record(record) {}
 
-    virtual void setup(OperationContext* opCtx) {
+    virtual void setup() {
         invariant(!_filesLock.get());
-        _filesLock.reset(new LockMongoFilesShared(opCtx));
+        _filesLock.reset(new LockMongoFilesShared());
     }
 
     virtual void fetch() {
@@ -117,19 +113,12 @@ MmapV1ExtentManager::MmapV1ExtentManager(StringData dbname, StringData path, boo
       _path(path.toString()),
       _directoryPerDB(directoryPerDB),
       _rid(RESOURCE_METADATA, dbname) {
-    StorageEngine* engine = getGlobalServiceContext()->getStorageEngine();
+    StorageEngine* engine = getGlobalServiceContext()->getGlobalStorageEngine();
     invariant(engine->isMmapV1());
     MMAPV1Engine* mmapEngine = static_cast<MMAPV1Engine*>(engine);
     _recordAccessTracker = &mmapEngine->getRecordAccessTracker();
 }
 
-std::unique_ptr<ExtentManager> MmapV1ExtentManager::Factory::create(StringData dbname,
-                                                                    StringData path,
-                                                                    bool directoryPerDB) {
-    return stdx::make_unique<MmapV1ExtentManager>(
-        std::move(dbname), std::move(path), directoryPerDB);
-}
-
 boost::filesystem::path MmapV1ExtentManager::_fileName(int n) const {
     stringstream ss;
     ss << _dbname << '.' << n;
@@ -141,7 +130,7 @@ boost::filesystem::path MmapV1ExtentManager::_fileName(int n) const {
 }
 
 
-Status MmapV1ExtentManager::init(OperationContext* opCtx) {
+Status MmapV1ExtentManager::init(OperationContext* txn) {
     invariant(_files.empty());
 
     for (int n = 0; n < DiskLoc::MaxFiles; n++) {
@@ -175,18 +164,17 @@ Status MmapV1ExtentManager::init(OperationContext* opCtx) {
             }
         }
 
-        unique_ptr<DataFile> df(new DataFile(opCtx, n));
+        unique_ptr<DataFile> df(new DataFile(n));
 
-        Status s = df->openExisting(opCtx, fullNameString.c_str());
+        Status s = df->openExisting(fullNameString.c_str());
         if (!s.isOK()) {
-            df->close(opCtx);
             return s;
         }
 
         invariant(!df->getHeader()->uninitialized());
 
         // We only checkUpgrade on files that we are keeping, not preallocs.
-        df->getHeader()->checkUpgrade(opCtx);
+        df->getHeader()->checkUpgrade(txn);
 
         _files.push_back(df.release());
     }
@@ -194,13 +182,13 @@ Status MmapV1ExtentManager::init(OperationContext* opCtx) {
     // If this is a new database being created, instantiate the first file and one extent so
     // we can have a coherent database.
     if (_files.empty()) {
-        WriteUnitOfWork wuow(opCtx);
-        _createExtent(opCtx, initialSize(128), false);
+        WriteUnitOfWork wuow(txn);
+        _createExtent(txn, initialSize(128), false);
         wuow.commit();
 
         // Commit the journal and all changes to disk so that even if exceptions occur during
         // subsequent initialization, we won't have uncommited changes during file close.
-        getDur().commitNow(opCtx);
+        getDur().commitNow(txn);
 
         audit::logCreateDatabase(&cc(), _dbname);
     }
@@ -209,25 +197,29 @@ Status MmapV1ExtentManager::init(OperationContext* opCtx) {
 }
 
 const DataFile* MmapV1ExtentManager::_getOpenFile(int fileId) const {
-    invariant(fileId >= 0 && fileId < _files.size(),
-              str::stream() << "_getOpenFile() invalid file index requested " << fileId);
+    if (fileId < 0 || fileId >= _files.size()) {
+        log() << "_getOpenFile() invalid file index requested " << fileId;
+        invariant(false);
+    }
 
     return _files[fileId];
 }
 
 DataFile* MmapV1ExtentManager::_getOpenFile(int fileId) {
-    invariant(fileId >= 0 && fileId < _files.size(),
-              str::stream() << "_getOpenFile() invalid file index requested " << fileId);
+    if (fileId < 0 || fileId >= _files.size()) {
+        log() << "_getOpenFile() invalid file index requested " << fileId;
+        invariant(false);
+    }
 
     return _files[fileId];
 }
 
-DataFile* MmapV1ExtentManager::_addAFile(OperationContext* opCtx,
+DataFile* MmapV1ExtentManager::_addAFile(OperationContext* txn,
                                          int sizeNeeded,
                                          bool preallocateNextFile) {
     // Database must be stable and we need to be in some sort of an update operation in order
     // to add a new file.
-    invariant(opCtx->lockState()->isDbLockedForMode(_dbname, MODE_IX));
+    invariant(txn->lockState()->isDbLockedForMode(_dbname, MODE_IX));
 
     const int allocFileId = _files.size();
 
@@ -242,17 +234,12 @@ DataFile* MmapV1ExtentManager::_addAFile(OperationContext* opCtx,
     }
 
     {
-        unique_ptr<DataFile> allocFile(new DataFile(opCtx, allocFileId));
+        unique_ptr<DataFile> allocFile(new DataFile(allocFileId));
         const string allocFileName = _fileName(allocFileId).string();
 
         Timer t;
 
-        try {
-            allocFile->open(opCtx, allocFileName.c_str(), minSize, false);
-        } catch (...) {
-            allocFile->close(opCtx);
-            throw;
-        }
+        allocFile->open(txn, allocFileName.c_str(), minSize, false);
         if (t.seconds() > 1) {
             log() << "MmapV1ExtentManager took " << t.seconds()
                   << " seconds to open: " << allocFileName;
@@ -264,15 +251,10 @@ DataFile* MmapV1ExtentManager::_addAFile(OperationContext* opCtx,
 
     // Preallocate is asynchronous
     if (preallocateNextFile) {
-        unique_ptr<DataFile> nextFile(new DataFile(opCtx, allocFileId + 1));
+        unique_ptr<DataFile> nextFile(new DataFile(allocFileId + 1));
         const string nextFileName = _fileName(allocFileId + 1).string();
 
-        try {
-            nextFile->open(opCtx, nextFileName.c_str(), minSize, false);
-        } catch (...) {
-            nextFile->close(opCtx);
-            throw;
-        }
+        nextFile->open(txn, nextFileName.c_str(), minSize, false);
     }
 
     // Returns the last file added
@@ -367,26 +349,26 @@ int MmapV1ExtentManager::maxSize() const {
 }
 
 DiskLoc MmapV1ExtentManager::_createExtentInFile(
-    OperationContext* opCtx, int fileNo, DataFile* f, int size, bool enforceQuota) {
+    OperationContext* txn, int fileNo, DataFile* f, int size, bool enforceQuota) {
     _checkQuota(enforceQuota, fileNo - 1);
 
     massert(10358, "bad new extent size", size >= minSize() && size <= maxSize());
 
-    DiskLoc loc = f->allocExtentArea(opCtx, size);
+    DiskLoc loc = f->allocExtentArea(txn, size);
     loc.assertOk();
 
     Extent* e = getExtent(loc, false);
     verify(e);
 
-    *opCtx->recoveryUnit()->writing(&e->magic) = Extent::extentSignature;
-    *opCtx->recoveryUnit()->writing(&e->myLoc) = loc;
-    *opCtx->recoveryUnit()->writing(&e->length) = size;
+    *txn->recoveryUnit()->writing(&e->magic) = Extent::extentSignature;
+    *txn->recoveryUnit()->writing(&e->myLoc) = loc;
+    *txn->recoveryUnit()->writing(&e->length) = size;
 
     return loc;
 }
 
 
-DiskLoc MmapV1ExtentManager::_createExtent(OperationContext* opCtx, int size, bool enforceQuota) {
+DiskLoc MmapV1ExtentManager::_createExtent(OperationContext* txn, int size, bool enforceQuota) {
     size = quantizeExtentSize(size);
 
     if (size > maxSize())
@@ -399,7 +381,7 @@ DiskLoc MmapV1ExtentManager::_createExtent(OperationContext* opCtx, int size, bo
         invariant(f);
 
         if (f->getHeader()->unusedLength >= size) {
-            return _createExtentInFile(opCtx, i, f, size, enforceQuota);
+            return _createExtentInFile(txn, i, f, size, enforceQuota);
         }
     }
 
@@ -408,10 +390,10 @@ DiskLoc MmapV1ExtentManager::_createExtent(OperationContext* opCtx, int size, bo
     // no space in an existing file
     // allocate files until we either get one big enough or hit maxSize
     for (int i = 0; i < 8; i++) {
-        DataFile* f = _addAFile(opCtx, size, false);
+        DataFile* f = _addAFile(txn, size, false);
 
         if (f->getHeader()->unusedLength >= size) {
-            return _createExtentInFile(opCtx, numFiles() - 1, f, size, enforceQuota);
+            return _createExtentInFile(txn, numFiles() - 1, f, size, enforceQuota);
         }
     }
 
@@ -419,7 +401,7 @@ DiskLoc MmapV1ExtentManager::_createExtent(OperationContext* opCtx, int size, bo
     msgasserted(14810, "couldn't allocate space for a new extent");
 }
 
-DiskLoc MmapV1ExtentManager::_allocFromFreeList(OperationContext* opCtx,
+DiskLoc MmapV1ExtentManager::_allocFromFreeList(OperationContext* txn,
                                                 int approxSize,
                                                 bool capped) {
     // setup extent constraints
@@ -494,27 +476,27 @@ DiskLoc MmapV1ExtentManager::_allocFromFreeList(OperationContext* opCtx,
 
     // remove from the free list
     if (!best->xprev.isNull())
-        *opCtx->recoveryUnit()->writing(&getExtent(best->xprev)->xnext) = best->xnext;
+        *txn->recoveryUnit()->writing(&getExtent(best->xprev)->xnext) = best->xnext;
     if (!best->xnext.isNull())
-        *opCtx->recoveryUnit()->writing(&getExtent(best->xnext)->xprev) = best->xprev;
+        *txn->recoveryUnit()->writing(&getExtent(best->xnext)->xprev) = best->xprev;
     if (_getFreeListStart() == best->myLoc)
-        _setFreeListStart(opCtx, best->xnext);
+        _setFreeListStart(txn, best->xnext);
     if (_getFreeListEnd() == best->myLoc)
-        _setFreeListEnd(opCtx, best->xprev);
+        _setFreeListEnd(txn, best->xprev);
 
     return best->myLoc;
 }
 
-DiskLoc MmapV1ExtentManager::allocateExtent(OperationContext* opCtx,
+DiskLoc MmapV1ExtentManager::allocateExtent(OperationContext* txn,
                                             bool capped,
                                             int size,
                                             bool enforceQuota) {
-    Lock::ResourceLock rlk(opCtx->lockState(), _rid, MODE_X);
+    Lock::ResourceLock rlk(txn->lockState(), _rid, MODE_X);
     bool fromFreeList = true;
-    DiskLoc eloc = _allocFromFreeList(opCtx, size, capped);
+    DiskLoc eloc = _allocFromFreeList(txn, size, capped);
     if (eloc.isNull()) {
         fromFreeList = false;
-        eloc = _createExtent(opCtx, size, enforceQuota);
+        eloc = _createExtent(txn, size, enforceQuota);
     }
 
     invariant(!eloc.isNull());
@@ -526,29 +508,29 @@ DiskLoc MmapV1ExtentManager::allocateExtent(OperationContext* opCtx,
     return eloc;
 }
 
-void MmapV1ExtentManager::freeExtent(OperationContext* opCtx, DiskLoc firstExt) {
-    Lock::ResourceLock rlk(opCtx->lockState(), _rid, MODE_X);
+void MmapV1ExtentManager::freeExtent(OperationContext* txn, DiskLoc firstExt) {
+    Lock::ResourceLock rlk(txn->lockState(), _rid, MODE_X);
     Extent* e = getExtent(firstExt);
-    opCtx->recoveryUnit()->writing(&e->xnext)->Null();
-    opCtx->recoveryUnit()->writing(&e->xprev)->Null();
-    opCtx->recoveryUnit()->writing(&e->firstRecord)->Null();
-    opCtx->recoveryUnit()->writing(&e->lastRecord)->Null();
+    txn->recoveryUnit()->writing(&e->xnext)->Null();
+    txn->recoveryUnit()->writing(&e->xprev)->Null();
+    txn->recoveryUnit()->writing(&e->firstRecord)->Null();
+    txn->recoveryUnit()->writing(&e->lastRecord)->Null();
 
 
     if (_getFreeListStart().isNull()) {
-        _setFreeListStart(opCtx, firstExt);
-        _setFreeListEnd(opCtx, firstExt);
+        _setFreeListStart(txn, firstExt);
+        _setFreeListEnd(txn, firstExt);
     } else {
         DiskLoc a = _getFreeListStart();
         invariant(getExtent(a)->xprev.isNull());
-        *opCtx->recoveryUnit()->writing(&getExtent(a)->xprev) = firstExt;
-        *opCtx->recoveryUnit()->writing(&getExtent(firstExt)->xnext) = a;
-        _setFreeListStart(opCtx, firstExt);
+        *txn->recoveryUnit()->writing(&getExtent(a)->xprev) = firstExt;
+        *txn->recoveryUnit()->writing(&getExtent(firstExt)->xnext) = a;
+        _setFreeListStart(txn, firstExt);
     }
 }
 
-void MmapV1ExtentManager::freeExtents(OperationContext* opCtx, DiskLoc firstExt, DiskLoc lastExt) {
-    Lock::ResourceLock rlk(opCtx->lockState(), _rid, MODE_X);
+void MmapV1ExtentManager::freeExtents(OperationContext* txn, DiskLoc firstExt, DiskLoc lastExt) {
+    Lock::ResourceLock rlk(txn->lockState(), _rid, MODE_X);
 
     if (firstExt.isNull() && lastExt.isNull())
         return;
@@ -564,14 +546,14 @@ void MmapV1ExtentManager::freeExtents(OperationContext* opCtx, DiskLoc firstExt,
     }
 
     if (_getFreeListStart().isNull()) {
-        _setFreeListStart(opCtx, firstExt);
-        _setFreeListEnd(opCtx, lastExt);
+        _setFreeListStart(txn, firstExt);
+        _setFreeListEnd(txn, lastExt);
     } else {
         DiskLoc a = _getFreeListStart();
         invariant(getExtent(a)->xprev.isNull());
-        *opCtx->recoveryUnit()->writing(&getExtent(a)->xprev) = lastExt;
-        *opCtx->recoveryUnit()->writing(&getExtent(lastExt)->xnext) = a;
-        _setFreeListStart(opCtx, firstExt);
+        *txn->recoveryUnit()->writing(&getExtent(a)->xprev) = lastExt;
+        *txn->recoveryUnit()->writing(&getExtent(lastExt)->xnext) = a;
+        _setFreeListStart(txn, firstExt);
     }
 }
 
@@ -589,22 +571,22 @@ DiskLoc MmapV1ExtentManager::_getFreeListEnd() const {
     return file->header()->freeListEnd;
 }
 
-void MmapV1ExtentManager::_setFreeListStart(OperationContext* opCtx, DiskLoc loc) {
+void MmapV1ExtentManager::_setFreeListStart(OperationContext* txn, DiskLoc loc) {
     invariant(!_files.empty());
     DataFile* file = _files[0];
-    *opCtx->recoveryUnit()->writing(&file->header()->freeListStart) = loc;
+    *txn->recoveryUnit()->writing(&file->header()->freeListStart) = loc;
 }
 
-void MmapV1ExtentManager::_setFreeListEnd(OperationContext* opCtx, DiskLoc loc) {
+void MmapV1ExtentManager::_setFreeListEnd(OperationContext* txn, DiskLoc loc) {
     invariant(!_files.empty());
     DataFile* file = _files[0];
-    *opCtx->recoveryUnit()->writing(&file->header()->freeListEnd) = loc;
+    *txn->recoveryUnit()->writing(&file->header()->freeListEnd) = loc;
 }
 
-void MmapV1ExtentManager::freeListStats(OperationContext* opCtx,
+void MmapV1ExtentManager::freeListStats(OperationContext* txn,
                                         int* numExtents,
                                         int64_t* totalFreeSizeBytes) const {
-    Lock::ResourceLock rlk(opCtx->lockState(), _rid, MODE_S);
+    Lock::ResourceLock rlk(txn->lockState(), _rid, MODE_S);
 
     invariant(numExtents);
     invariant(totalFreeSizeBytes);
@@ -621,6 +603,19 @@ void MmapV1ExtentManager::freeListStats(OperationContext* opCtx,
     }
 }
 
+void MmapV1ExtentManager::printFreeList() const {
+    log() << "dump freelist " << _dbname << endl;
+
+    DiskLoc a = _getFreeListStart();
+    while (!a.isNull()) {
+        Extent* e = getExtent(a);
+        log() << "  extent " << a.toString() << " len:" << e->length
+              << " prev:" << e->xprev.toString() << endl;
+        a = e->xnext;
+    }
+
+    log() << "end freelist" << endl;
+}
 
 namespace {
 class CacheHintMadvise : public ExtentManager::CacheHint {
@@ -645,12 +640,6 @@ MmapV1ExtentManager::FilesArray::~FilesArray() {
     }
 }
 
-void MmapV1ExtentManager::FilesArray::close(OperationContext* opCtx) {
-    for (int i = 0; i < size(); i++) {
-        _files[i]->close(opCtx);
-    }
-}
-
 void MmapV1ExtentManager::FilesArray::push_back(DataFile* val) {
     stdx::lock_guard<stdx::mutex> lk(_writersMutex);
     const int n = _size.load();
@@ -660,7 +649,7 @@ void MmapV1ExtentManager::FilesArray::push_back(DataFile* val) {
     _size.store(n + 1);
 }
 
-DataFileVersion MmapV1ExtentManager::getFileFormat(OperationContext* opCtx) const {
+DataFileVersion MmapV1ExtentManager::getFileFormat(OperationContext* txn) const {
     if (numFiles() == 0)
         return DataFileVersion(0, 0);
 
@@ -668,12 +657,12 @@ DataFileVersion MmapV1ExtentManager::getFileFormat(OperationContext* opCtx) cons
     return _getOpenFile(0)->getHeader()->version;
 }
 
-void MmapV1ExtentManager::setFileFormat(OperationContext* opCtx, DataFileVersion newVersion) {
+void MmapV1ExtentManager::setFileFormat(OperationContext* txn, DataFileVersion newVersion) {
     invariant(numFiles() > 0);
 
     DataFile* df = _getOpenFile(0);
     invariant(df);
 
-    *opCtx->recoveryUnit()->writing(&df->getHeader()->version) = newVersion;
+    *txn->recoveryUnit()->writing(&df->getHeader()->version) = newVersion;
 }
 }
