@@ -111,7 +111,7 @@ namespace mongo {
                 return v;
             }
 
-            void append(BSONObjBuilder& builder, const StringData& name, int scale = 1) const {
+            void append(BSONObjBuilder& builder, StringData name, int scale = 1) const {
                 if (_type == FilesystemState) {
                     BSONObjBuilder fsBuilder(builder.subobjStart(name));
                     if (_fsState == FS_GREEN) {
@@ -155,7 +155,7 @@ namespace mongo {
             }
 
             uint64_t getInteger() const {
-                massert(28617, "PerconaFT: wrong engine status type for getInteger", _type == UInt64);
+                massert(28930, "PerconaFT: wrong engine status type for getInteger", _type == UInt64);
                 return _uint64;
             }
 
@@ -165,7 +165,7 @@ namespace mongo {
                 } else if (_type == UnixTime) {
                     return static_cast<double>(_time);
                 }
-                msgasserted(28591, "PerconaFT: wrong engine status type for getDouble");
+                msgasserted(28931, "PerconaFT: wrong engine status type for getDouble");
             }
         };
 
@@ -212,7 +212,7 @@ namespace mongo {
             operator BSONObjBuilder&() { return b(); }
         };
 
-        NestedBuilder(Stack &stack, const StringData &name)
+        NestedBuilder(Stack &stack, StringData name)
             : _stack(stack), _b(_stack.b().subobjStart(name))
         {
             _stack.push(&_b);
@@ -245,7 +245,7 @@ namespace mongo {
                 if (scaleElt.ok()) {
                     scale = scaleElt.safeNumberLong();
                 }
-                uassert(28608, "scale must be positive", scale > 0);
+                uassert(28932, "scale must be positive", scale > 0);
             }
 
             NestedBuilder::Stack result;
