@@ -387,12 +387,12 @@ var ReplSetTest = function(opts) {
         return this.name + "/" + hosts.join(",");
     };
 
-    this.startSet = function(options) {
+    this.startSet = function(options, restart) {
         print("ReplSetTest starting set");
 
         var nodes = [];
         for (var n = 0; n < this.ports.length; n++) {
-            nodes.push(this.start(n, options));
+            nodes.push(this.start(n, options, restart));
         }
 
         this.nodes = nodes;
@@ -967,7 +967,14 @@ var ReplSetTest = function(opts) {
             options.binVersion = MongoRunner.versionIterator(options.binVersion);
         }
 
-        options = Object.merge(defaults, options);
+        // If restarting a node, use its existing options as the defaults.
+        if (((options && options.restart) || restart) && this.nodes[n] &&
+            this.nodes[n].hasOwnProperty("fullOptions")) {
+            options = Object.merge(this.nodes[n].fullOptions, options);
+        } else {
+            options = Object.merge(defaults, options);
+        }
+
         options = Object.merge(options, this.nodeOptions["n" + n]);
         delete options.rsConfig;
 
@@ -1326,7 +1333,15 @@ var ReplSetTest = function(opts) {
      * Constructor, which instantiates the ReplSetTest object from an existing set.
      */
     function _constructFromExistingSeedNode(seedNode) {
-        var conf = _replSetGetConfig(new Mongo(seedNode));
+        const conn = new Mongo(seedNode);
+        var conf;
+        if (jsTest.options().keyFile) {
+            self.keyFile = jsTest.options().keyFile;
+            conf = authutil.asCluster(conn, self.keyFile, () => _replSetGetConfig(conn));
+        } else {
+            conf = _replSetGetConfig(conn);
+        }
+
         print('Recreating replica set from config ' + tojson(conf));
 
         var existingNodes = conf.members.map(member => member.host);
